==> redpesk-cli_3.15.0.orig.tar.gz: redpesk-cli-3.15.0/cmd/iotpkg_wrap.go <==
/*
Copyright © 2020-2023 IoT.bzh Company
Contact: https://www.iot.bzh/licensing

This file is part of the redpesk-cli project, the redpesk command line tool.

$RP_BEGIN_LICENSE$
Commercial License Usage
 Licensees holding valid commercial IoT.bzh licenses may use this file in
 accordance with the commercial license agreement provided with the
 Software or, alternatively, in accordance with the terms contained in
 a written agreement between you and The IoT.bzh Company. For licensing terms
 and conditions see https://www.iot.bzh/terms-conditions. For further
 information use the contact form at https://www.iot.bzh/contact.

GNU General Public License Usage
 Alternatively, this file may be used under the terms of the GNU General
 Public license version 3. This license is as published by the Free Software
 Foundation and appearing in the file LICENSE.GPLv3 included in the packaging
 of this file. Please review the following information to ensure the GNU
 General Public License requirements will be met
 https://www.gnu.org/licenses/gpl-3.0.html.
$RP_END_LICENSE$
*/
package cmd

import (
	"errors"
	"fmt"
	"os"
	"strings"

	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/rpxxxapi/ladapi"
	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/utils"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"
	"golang.org/x/term"
)

// --------------------------------------------------------------
// Initialization functions -------------------------------------
// --------------------------------------------------------------

func initIotpkgCmd(rootCmd *cobra.Command) {

	// localCmd represents the local command
	var iotpkgCmd = &cobra.Command{
		Use:                iotpkgCommand,
		Short:              iotpkgCmdShortDesc,
		Long:               iotpkgCmdLongDesc,
		SilenceUsage:       true,
		DisableFlagParsing: true,
		PreRunE:            initEventsRPLAD,
		RunE:               iotpkgWrapperCmd,
		Annotations:        map[string]string{"__bck_connect__": "FALSE"}, // No need for backend connection for this command
	}

	// Here you will define your flags and configuration settings.

	// Persistent Flags -----------------------------------------
	// ----------------------------------------------------------

	// Bind the cobra flags to viper
	// ----------------------------------------------------------

	// Local Flags ----------------------------------------------
	// ----------------------------------------------------------

	// Add some flags as required

	// Add some flags as hidden

	// Specify dynamic flag completion

	// Add the command "iotpkg" as a subcommand of rp-cli
	rootCmd.AddCommand(iotpkgCmd)
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Completion functions -----------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Command functions --------------------------------------------
// --------------------------------------------------------------
// iotpkgWrapperCmd runs the iotpkg command line inside the local-builder
// configured on the host
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went wrong, nil in the other case
func iotpkgWrapperCmd(cmd *cobra.Command, args []string) error {

	// Get the command line passed by the user
	cmdLine, err := getCmdLine(args)
	if err != nil {
		return err
	}

	// Add the `iotpkg` command in front of what we retrieve
	cmdLine = "iotpkg " + cmdLine

	// Get the size of the current terminal
	termWidth, termHeight, err := term.GetSize(int(os.Stdin.Fd()))
	if err != nil {
		return err
	}

	// Get the current working directory
	currentDir, err := os.Getwd()
	if err != nil {
		return err
	}

	// Check that we are in a gitpkgs/ subdirectory
	tmpPkgsDir, err := manuallyGetPkgDirFlag()
	if err != nil {
		return err
	}

	// Check that the flag has been set
	if tmpPkgsDir == "" {
		return errors.New(hostPkgDirNotSetMsg)
	}

	pkgsDir, err := utils.SanitizePath(tmpPkgsDir)
	if err != nil {
		return err
	}
	isInPkgsDir := strings.HasPrefix(currentDir, pkgsDir)
	if !isInPkgsDir && (runMode != "testing") {
		errorMsg := fmt.Sprintf(userNotInGitpkgsDirMsg, pkgsDir)
		return errors.New(errorMsg)
	}

	// Convert the current working dir to the one inside local-builder
	localBuildWorkDir := strings.ReplaceAll(currentDir, pkgsDir, localBuilderGitPkgsDir)

	// Request local-builder to run the command
	// Request body
	execRequest := ladapi.RPLADExecCmdStruct{
		RPLADMainCmdParams: ladapi.RPLADMainCmdParams{
			RPLADWinSize: ladapi.RPLADWinSize{
				WinSizeRows: termHeight,
				WinSizeCol:  termWidth,
			},
			WorkingDir: localBuildWorkDir,
		},
		CmdLine: cmdLine,
	}
	// Answer body template
	procTicket := ladapi.RPLADRunnerTicket{}
	err = localBAPI.ExecPost(&execRequest, &procTicket)
	if err != nil {
		return err
	}

	err = listenStdoutErrAndIn(procTicket)

	return err
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Utilities functions ------------------------------------------
// --------------------------------------------------------------

// manuallyGetPkgDirFlag is a function allowing to manually get the flag
// "pkgsdir" that needs to be defined in the configuration file.
//
// :returns: string, error -- the value of the flag and an error if something went wrong
func manuallyGetPkgDirFlag() (string, error) {
	pkgsDir, err := manuallyGetFlagInConfigFile(localPkgCmdPackageFlag)
	return pkgsDir, err
}

// manuallyGetSrcDirFlag is a function allowing to manually get the flag
// "sourcesdir" that needs to be defined in the configuration file.
//
// :returns: string, error -- the value of the flag and an error if something went wrong
func manuallyGetSrcDirFlag() (string, error) {
	srcDir, err := manuallyGetFlagInConfigFile(localPkgCmdSourceFlag)
	return srcDir, err
}

// manuallyGetFlagInConfigFile is a function allowing to manually get a flag
// defined in the configuration file
//
// :param flagName: The name of the flag to look for
// :type flagName: string
// :returns: string, error -- the value of the flag and an error if something went wrong
func manuallyGetFlagInConfigFile(flagName string) (string, error) {
	paramValue := ""

	// First thing to do, check if the flag is defined in the connection file for the current server alias
	// Get the alias of the remote connection
	serverAlias := viper.GetString(serverAliasFlag)

	// Get the connection parameters
	connecParams, err := getConnectionParams(serverAlias, accessTokenFile)
	if err != nil {
		return "", err
	}

	// Look for the right field in the connection
	if flagName == localPkgCmdSourceFlag {
		paramValue = connecParams.SourcesDir
	} else if flagName == localPkgCmdPackageFlag {
		paramValue = connecParams.PkgsDir
	}

	if paramValue != "" {
		return paramValue, nil
	}

	// Get the path to the configuration file used
	configFilePath := viper.ConfigFileUsed()
	if configFilePath == "" {
		return "", errors.New(configFileNotFound)
	}

	// Get the value of the "pkgsdir" flag from the configuration file
	configJSON := make(map[string]interface{})
	err = utils.ReadJsonFile(configFilePath, &configJSON)
	if err != nil {
		return "", err
	}

	if tmpParamValue, ok := configJSON[flagName]; ok {
		paramValue = fmt.Sprintf("%v", tmpParamValue)
	}

	return paramValue, nil
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Display functions --------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// --------------------------------------------------------------

==> redpesk-cli_3.18.0.orig.tar.gz: redpesk-cli-3.18.0/cmd/iotpkg_wrap.go <==
/*
Copyright © 2020-2023 IoT.bzh Company
Contact: https://www.iot.bzh/licensing

This file is part of the redpesk-cli project, the redpesk command line tool.

$RP_BEGIN_LICENSE$
Commercial License Usage

	Licensees holding valid commercial IoT.bzh licenses may use this file in
	accordance with the commercial license agreement provided with the
	Software or, alternatively, in accordance with the terms contained in
	a written agreement between you and The IoT.bzh Company. For licensing terms
	and conditions see https://www.iot.bzh/terms-conditions. For further
	information use the contact form at https://www.iot.bzh/contact.

GNU General Public License Usage

	Alternatively, this file may be used under the terms of the GNU General
	Public license version 3. This license is as published by the Free Software
	Foundation and appearing in the file LICENSE.GPLv3 included in the packaging
	of this file. Please review the following information to ensure the GNU
	General Public License requirements will be met
	https://www.gnu.org/licenses/gpl-3.0.html.

$RP_END_LICENSE$
*/
package cmd

import (
	"errors"
	"fmt"
	"os"
	"strings"

	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/rpxxxapi/ladapi"
	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/utils"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"
	"golang.org/x/term"
)

// --------------------------------------------------------------
// Initialization functions -------------------------------------
// --------------------------------------------------------------

func initIotpkgCmd(rootCmd *cobra.Command) {

	// localCmd represents the local command
	var iotpkgCmd = &cobra.Command{
		Use:                iotpkgCommand,
		Short:              iotpkgCmdShortDesc,
		Long:               iotpkgCmdLongDesc,
		SilenceUsage:       true,
		DisableFlagParsing: true,
		PreRunE:            initEventsRPLAD,
		RunE:               iotpkgWrapperCmd,
		Annotations: map[string]string{
			"__bck_connect__": "FALSE", // No need for backend connection for this command
			"__lad_connect__": "TRUE"},
	}

	// Here you will define your flags and configuration settings.

	// Persistent Flags -----------------------------------------
	// ----------------------------------------------------------

	// Bind the cobra flags to viper
	// ----------------------------------------------------------

	// Local Flags ----------------------------------------------
	// ----------------------------------------------------------

	// Add some flags as required

	// Add some flags as hidden

	// Specify dynamic flag completion

	// Add the command "iotpkg" as a subcommand of rp-cli
	rootCmd.AddCommand(iotpkgCmd)
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Completion functions -----------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Command functions --------------------------------------------
// --------------------------------------------------------------
// iotpkgWrapperCmd runs the iotpkg command line inside the local-builder
// configured on the host
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went wrong, nil in the other case
func iotpkgWrapperCmd(cmd *cobra.Command, args []string) error {

	// Get the command line passed by the user
	cmdLine, err := getCmdLine(args)
	if err != nil {
		return err
	}

	// Add the `iotpkg` command in front of what we retrieve
	cmdLine = "iotpkg " + cmdLine

	// Get the size of the current terminal
	termWidth, termHeight, err := term.GetSize(int(os.Stdin.Fd()))
	if err != nil {
		return err
	}

	// Get the current working directory
	currentDir, err := os.Getwd()
	if err != nil {
		return err
	}

	// Check that we are in a gitpkgs/ subdirectory
	tmpPkgsDir, err := manuallyGetPkgDirFlag()
	if err != nil {
		return err
	}

	// Check that the flag has been set
	if tmpPkgsDir == "" {
		return errors.New(hostPkgDirNotSetMsg)
	}

	pkgsDir, err := utils.SanitizePath(tmpPkgsDir)
	if err != nil {
		return err
	}
	isInPkgsDir := strings.HasPrefix(currentDir, pkgsDir)
	if !isInPkgsDir && (runMode != "testing") {
		errorMsg := fmt.Sprintf(userNotInGitpkgsDirMsg, pkgsDir)
		return errors.New(errorMsg)
	}

	// Convert the current working dir to the one inside local-builder
	localBuildWorkDir := strings.ReplaceAll(currentDir, pkgsDir, localBuilderGitPkgsDir)

	// Request local-builder to run the command
	// Request body
	execRequest := ladapi.RPLADExecCmdStruct{
		RPLADMainCmdParams: ladapi.RPLADMainCmdParams{
			RPLADWinSize: ladapi.RPLADWinSize{
				WinSizeRows: termHeight,
				WinSizeCol:  termWidth,
			},
			WorkingDir: localBuildWorkDir,
		},
		CmdLine: cmdLine,
	}
	// Answer body template
	procTicket := ladapi.RPLADRunnerTicket{}
	err = localBAPI.ExecPost(&execRequest, &procTicket)
	if err != nil {
		return err
	}

	err = listenStdoutErrAndIn(procTicket)

	return err
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Utilities functions ------------------------------------------
// --------------------------------------------------------------

// manuallyGetPkgDirFlag is a function allowing to manually get the flag
// "pkgsdir" that needs to be defined in the configuration file.
//
// :returns: string, error -- the value of the flag and an error if something went wrong
func manuallyGetPkgDirFlag() (string, error) {
	pkgsDir, err := manuallyGetFlagInConfigFile(localPkgCmdPackageFlag)
	return pkgsDir, err
}

// manuallyGetSrcDirFlag is a function allowing to manually get the flag
// "sourcesdir" that needs to be defined in the configuration file.
//
// :returns: string, error -- the value of the flag and an error if something went wrong
func manuallyGetSrcDirFlag() (string, error) {
	srcDir, err := manuallyGetFlagInConfigFile(localPkgCmdSourceFlag)
	return srcDir, err
}

// manuallyGetFlagInConfigFile is a function allowing to manually get a flag
// defined in the configuration file
//
// :param flagName: The name of the flag to look for
// :type flagName: string
// :returns: string, error -- the value of the flag and an error if something went wrong
func manuallyGetFlagInConfigFile(flagName string) (string, error) {
	paramValue := ""

	// First thing to do, check if the flag is defined in the connection file for the current server alias
	// Get the alias of the remote connection
	serverAlias := viper.GetString(serverAliasFlag)

	// Get the connection parameters
	connecParams, err := getConnectionParams(serverAlias, accessTokenFile)
	if err != nil {
		return "", err
	}

	// Look for the right field in the connection
	if flagName == localPkgCmdSourceFlag {
		paramValue = connecParams.SourcesDir
	} else if flagName == localPkgCmdPackageFlag {
		paramValue = connecParams.PkgsDir
	}

	if paramValue != "" {
		return paramValue, nil
	}

	// Get the path to the configuration file used
	configFilePath := viper.ConfigFileUsed()
	if configFilePath == "" {
		return "", errors.New(configFileNotFound)
	}

	// Get the value of the "pkgsdir" flag from the configuration file
	configJSON := make(map[string]interface{})
	err = utils.ReadJsonFile(configFilePath, &configJSON)
	if err != nil {
		return "", err
	}

	if tmpParamValue, ok := configJSON[flagName]; ok {
		paramValue = fmt.Sprintf("%v", tmpParamValue)
	}

	return paramValue, nil
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Display functions --------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// --------------------------------------------------------------
